@@ -31,6 +31,15 @@ class ExportStrategy(str, enum.Enum):
 class Algorithm(str, enum.Enum):
     """Different algorithms supported by Pynguin."""
 
+    DYNAMOSA = "DYNAMOSA"
+    """The dynamic many-objective sorting algorithm (cf. Panichella et al. Automated
+    test case generation as a many-objective optimisation problem with dynamic selection
+    of the targets.  TSE vol. 44 issue 2)."""
+
+    MIO = "MIO"
+    """The MIO test suite generation algorithm (cf. Andrea Arcuri. Many Independent
+    Objective (MIO) Algorithm for Test Suite Generation.  Proc. SBSE 2017)."""
+
     MOSA = "MOSA"
     """The many-objective sorting algorithm (cf. Panichella et al. Reformulating Branch
     Coverage as a Many-Objective Optimization Problem.  Proc. ICST 2015)."""
@@ -40,9 +49,12 @@ class Algorithm(str, enum.Enum):
     proposed by Randoop (cf. Pacheco et al. Feedback-directed random test generation.
     Proc. ICSE 2007)."""
 
-    RANDOM_SEARCH = "RANDOM_SEARCH"
+    RANDOM_TEST_SUITE_SEARCH = "RANDOM_TEST_SUITE_SEARCH"
     """Performs random search on test suites."""
 
+    RANDOM_TEST_CASE_SEARCH = "RANDOM_TEST_CASE_SEARCH"
+    """Performs random search on test cases."""
+
     WHOLE_SUITE = "WHOLE_SUITE"
     """A whole-suite test generation approach similar to the one proposed by EvoSuite
     (cf. Fraser and Arcuri. EvoSuite: Automatic Test Suite Generation for
@@ -88,6 +100,16 @@ class StatisticsBackend(str, enum.Enum):
     """Write statistics to a CSV file."""
 
 
+class Selection(str, enum.Enum):
+    """Different selection algorithms to select from."""
+
+    RANK_SELECTION = "RANK_SELECTION"
+    """Rank selection."""
+
+    TOURNAMENT_SELECTION = "TOURNAMENT_SELECTION"
+    """Tournament selection.  Use `tournament_size` to set size."""
+
+
 # pylint: disable=too-many-instance-attributes, pointless-string-statement
 @dataclasses.dataclass
 class Configuration:
@@ -259,9 +281,40 @@ class Configuration:
     statement_insertion_probability: float = 0.5
     """Initial probability of inserting a new statement in a test case"""
 
+    skip_optional_parameter_probability: float = 0.7
+    """Probability to skip an optional parameter, i.e., do not fill this parameter."""
+
     max_size: int = 100
     """Maximum number of test cases in a test suite"""
 
+    number_of_tests_per_target_initial: int = 10
+    """Number of test cases for each target goal to keep in an archive before focused
+    phase. Used for MIO."""
+
+    number_of_tests_per_target_focused: int = 1
+    """Number of test cases for each target goal to keep in an archive in focused
+    phase. Used for MIO."""
+
+    random_test_or_from_archive_probability_initial: float = 0.5
+    """Probability [0,1] of sampling a new test at random or choose an existing one in
+    an archive before focused phase. Used for MIO."""
+
+    random_test_or_from_archive_probability_focused: float = 0.0
+    """Probability [0,1] of sampling a new test at random or choose an existing one in
+    an archive in focused phase. Used for MIO."""
+
+    num_mutations_initial: int = 1
+    """Number of mutations allowed to be done on the same individual before
+    sampling a new one before focused phase. Used for MIO."""
+
+    num_mutations_focused: int = 10
+    """Number of mutations allowed to be done on the same individual before
+    sampling a new one in focused phase. Used for MIO."""
+
+    exploitation_starts_at_percent: float = 0.5
+    """Percentage ]0,1] of search budget after which exploitation is activated, i.e.,
+    switching to focused phase. Used for MIO."""
+
     stopping_condition: StoppingCondition = StoppingCondition.MAX_TIME
     """What condition should be checked to end the search/test generation."""
 
@@ -271,7 +324,7 @@ class Configuration:
     stub_dir: Optional[str] = None
     """Path to the pyi-stub files for the StubInferenceStrategy"""
 
-    constant_seeding: bool = False
+    constant_seeding: bool = True
     """Should the generator use a static constant seeding technique to improve constant
     generation?"""
 
@@ -301,17 +354,23 @@ class Configuration:
     float_precision: float = 0.01
     """Precision to use in float comparisons and assertions"""
 
-    dynamic_constant_seeding: bool = False
+    dynamic_constant_seeding: bool = True
     """Enables seeding of constants at runtime."""
 
-    seeded_primitives_reuse_probability: float = 0.9
+    seeded_primitives_reuse_probability: float = 0.2
     """Probability for using seeded primitive values instead of randomly
     generated ones."""
 
-    seeded_dynamic_values_reuse_probability: float = 0.9
+    seeded_dynamic_values_reuse_probability: float = 0.6
     """Probability of using dynamically seeded values when a primitive seeded
      value will be used."""
 
+    tournament_size: int = 5
+    """Number of individuals for tournament selection."""
+
+    selection: Selection = Selection.TOURNAMENT_SELECTION
+    """The selection operator for genetic algorithms."""
+
 
 # Singleton instance of the configuration.
 configuration = Configuration(
