@@ -8,7 +8,7 @@
 from __future__ import annotations
 
 import logging
-from typing import List, Optional, Set, Type, cast
+from typing import Any, Dict, List, Optional, Set, Type, cast
 
 from typing_inspect import get_args, get_origin
 
@@ -29,9 +29,9 @@ from pynguin.utils.generic.genericaccessibleobject import GenericAccessibleObjec
 from pynguin.utils.type_utils import (
     is_assignable_to,
     is_collection_type,
+    is_optional_parameter,
     is_primitive_type,
     is_type_unknown,
-    should_skip_parameter,
 )
 
 
@@ -194,7 +194,7 @@ class TestFactory:
         signature = constructor.inferred_signature
         length = test_case.size()
         try:
-            parameters: List[vr.VariableReference] = self.satisfy_parameters(
+            parameters: Dict[str, vr.VariableReference] = self.satisfy_parameters(
                 test_case=test_case,
                 signature=signature,
                 position=position,
@@ -261,7 +261,7 @@ class TestFactory:
                 test_case, method.owner, position, recursion_depth, allow_none=False
             )
         assert callee, "The callee must not be None"
-        parameters: List[vr.VariableReference] = self.satisfy_parameters(
+        parameters: Dict[str, vr.VariableReference] = self.satisfy_parameters(
             test_case=test_case,
             signature=signature,
             position=position,
@@ -365,7 +365,7 @@ class TestFactory:
 
         signature = function.inferred_signature
         length = test_case.size()
-        parameters: List[vr.VariableReference] = self.satisfy_parameters(
+        parameters: Dict[str, vr.VariableReference] = self.satisfy_parameters(
             test_case=test_case,
             signature=signature,
             position=position,
@@ -749,7 +749,7 @@ class TestFactory:
     @staticmethod
     def _get_reuse_parameters(
         test_case: tc.TestCase, inf_signature: InferredSignature, position: int
-    ) -> List[vr.VariableReference]:
+    ) -> Dict[str, vr.VariableReference]:
         """Find specified parameters from existing objects.
 
         Args:
@@ -758,14 +758,19 @@ class TestFactory:
             position: The position
 
         Returns:
-            A list of existing objects
+            A dict of existing objects
         """
-        found = []
+        found = {}
         for parameter_name, parameter_type in inf_signature.parameters.items():
-            if should_skip_parameter(inf_signature, parameter_name):
+            if (
+                is_optional_parameter(inf_signature, parameter_name)
+                and randomness.next_float()
+                < config.configuration.skip_optional_parameter_probability
+            ):
                 continue
-            assert parameter_type
-            found.append(test_case.get_random_object(parameter_type, position))
+            found[parameter_name] = test_case.get_random_object(
+                parameter_type, position
+            )
         return found
 
     @staticmethod
@@ -843,7 +848,7 @@ class TestFactory:
         recursion_depth: int = 0,
         allow_none: bool = True,
         can_reuse_existing_variables: bool = True,
-    ) -> List[vr.VariableReference]:
+    ) -> Dict[str, vr.VariableReference]:
         """Satisfy a list of parameters by reusing or creating variables.
 
         Args:
@@ -857,7 +862,7 @@ class TestFactory:
                 be reused.
 
         Returns:
-            A list of variable references for the parameters
+            A dict of variable references for the parameters
 
         Raises:
             ConstructionFailedException: if construction of an object failed
@@ -865,7 +870,7 @@ class TestFactory:
         if position < 0:
             position = test_case.size()
 
-        parameters: List[vr.VariableReference] = []
+        parameters: Dict[str, vr.VariableReference] = {}
         self._logger.debug(
             "Trying to satisfy %d parameters at position %d",
             len(signature.parameters),
@@ -877,10 +882,11 @@ class TestFactory:
 
             previous_length = test_case.size()
 
-            if should_skip_parameter(signature, parameter_name):
-                # TODO Implement generation for positional parameters of variable length
-                # TODO Implement generation for keyword parameters of variable length
-                self._logger.debug("Skip parameter %s", parameter_name)
+            if (
+                is_optional_parameter(signature, parameter_name)
+                and randomness.next_float()
+                < config.configuration.skip_optional_parameter_probability
+            ):
                 continue
 
             if can_reuse_existing_variables:
@@ -913,7 +919,7 @@ class TestFactory:
                     ),
                 )
 
-            parameters.append(var)
+            parameters[parameter_name] = var
             current_length = test_case.size()
             position += current_length - previous_length
 
@@ -1166,19 +1172,15 @@ class TestFactory:
         position: int,
         recursion_depth: int,
     ) -> vr.VariableReference:
-        if get_origin(parameter_type) == list:
-            return self._create_list(
+        if parameter_type in (list, set) or get_origin(parameter_type) in (list, set):
+            return self._create_list_or_set(
                 test_case, parameter_type, position, recursion_depth
             )
-        if get_origin(parameter_type) == set:
-            return self._create_set(
-                test_case, parameter_type, position, recursion_depth
-            )
-        if get_origin(parameter_type) == tuple:
+        if parameter_type == tuple or get_origin(parameter_type) == tuple:
             return self._create_tuple(
                 test_case, parameter_type, position, recursion_depth
             )
-        if get_origin(parameter_type) == dict:
+        if parameter_type == dict or get_origin(parameter_type) == dict:
             return self._create_dict(
                 test_case, parameter_type, position, recursion_depth
             )
@@ -1186,7 +1188,7 @@ class TestFactory:
 
     # TODO(fk) Methods below should be refactored asap,
     # as they contain a lot of duplicate code
-    def _create_list(
+    def _create_list_or_set(
         self,
         test_case: tc.TestCase,
         parameter_type: Type,
@@ -1194,47 +1196,26 @@ class TestFactory:
         recursion_depth: int,
     ) -> vr.VariableReference:
         args = get_args(parameter_type)
-        if len(args) != 1:
-            raise ConstructionFailedException()
+        if len(args) == 1:
+            element_type = args[0]
+        else:
+            element_type = Any
         size = randomness.next_int(0, config.configuration.collection_size)
         elements = []
         for _ in range(size):
             previous_length = test_case.size()
             var = self._create_or_reuse_variable(
-                test_case, args[0], position, recursion_depth + 1, True
+                test_case, element_type, position, recursion_depth + 1, True
             )
             if var is not None:
                 elements.append(var)
             position += test_case.size() - previous_length
-        ret = test_case.add_statement(
-            coll_stmt.ListStatement(test_case, parameter_type, elements), position
-        )
-        ret.distance = recursion_depth
-        return ret
-
-    def _create_set(
-        self,
-        test_case: tc.TestCase,
-        parameter_type: Type,
-        position: int,
-        recursion_depth: int,
-    ) -> vr.VariableReference:
-        args = get_args(parameter_type)
-        if len(args) != 1:
-            raise ConstructionFailedException()
-        size = randomness.next_int(0, config.configuration.collection_size)
-        elements = []
-        for _ in range(size):
-            previous_length = test_case.size()
-            var = self._create_or_reuse_variable(
-                test_case, args[0], position, recursion_depth + 1, True
-            )
-            if var is not None:
-                elements.append(var)
-            position += test_case.size() - previous_length
-        ret = test_case.add_statement(
-            coll_stmt.SetStatement(test_case, parameter_type, elements), position
+        collection_stmt = (
+            coll_stmt.ListStatement(test_case, parameter_type, elements)
+            if parameter_type == list or get_origin(parameter_type) == list
+            else coll_stmt.SetStatement(test_case, parameter_type, elements)
         )
+        ret = test_case.add_statement(collection_stmt, position)
         ret.distance = recursion_depth
         return ret
 
@@ -1276,19 +1257,23 @@ class TestFactory:
         recursion_depth: int,
     ) -> vr.VariableReference:
         args = get_args(parameter_type)
-        if len(args) != 2:
-            raise ConstructionFailedException()
+        if len(args) == 2:
+            key_type = args[0]
+            value_type = args[1]
+        else:
+            key_type = Any
+            value_type = Any
         size = randomness.next_int(0, config.configuration.collection_size)
         elements = []
         for _ in range(size):
             previous_length = test_case.size()
             key = self._create_or_reuse_variable(
-                test_case, args[0], position, recursion_depth + 1, True
+                test_case, key_type, position, recursion_depth + 1, True
             )
             position += test_case.size() - previous_length
             previous_length = test_case.size()
             value = self._create_or_reuse_variable(
-                test_case, args[1], position, recursion_depth + 1, True
+                test_case, value_type, position, recursion_depth + 1, True
             )
             position += test_case.size() - previous_length
             if key is not None and value is not None:
